@@ -16,6 +16,8 @@ package config
 import (
 	"bytes"
 	"crypto/tls"
+	"encoding/json"
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -23,9 +25,8 @@ import (
 	"strings"
 	"testing"
 
-	"encoding/json"
-
-	"gopkg.in/yaml.v2"
+	"github.com/stretchr/testify/require"
+	"go.yaml.in/yaml/v2"
 )
 
 // LoadTLSConfig parses the given file into a tls.Config.
@@ -59,32 +60,40 @@ var expectedTLSConfigs = []struct {
 	{
 		filename: "tls_config.empty.good.json",
 		config:   &tls.Config{},
-	}, {
+	},
+	{
 		filename: "tls_config.insecure.good.json",
 		config:   &tls.Config{InsecureSkipVerify: true},
-	}, {
+	},
+	{
 		filename: "tls_config.tlsversion.good.json",
 		config:   &tls.Config{MinVersion: tls.VersionTLS11},
-	}, {
+	},
+	{
 		filename: "tls_config.max_version.good.json",
 		config:   &tls.Config{MaxVersion: tls.VersionTLS12},
 	},
 	{
 		filename: "tls_config.empty.good.yml",
 		config:   &tls.Config{},
-	}, {
+	},
+	{
 		filename: "tls_config.insecure.good.yml",
 		config:   &tls.Config{InsecureSkipVerify: true},
-	}, {
+	},
+	{
 		filename: "tls_config.tlsversion.good.yml",
 		config:   &tls.Config{MinVersion: tls.VersionTLS11},
-	}, {
+	},
+	{
 		filename: "tls_config.max_version.good.yml",
 		config:   &tls.Config{MaxVersion: tls.VersionTLS12},
-	}, {
+	},
+	{
 		filename: "tls_config.max_and_min_version.good.yml",
 		config:   &tls.Config{MaxVersion: tls.VersionTLS12, MinVersion: tls.VersionTLS11},
-	}, {
+	},
+	{
 		filename: "tls_config.max_and_min_version_same.good.yml",
 		config:   &tls.Config{MaxVersion: tls.VersionTLS12, MinVersion: tls.VersionTLS12},
 	},
@@ -93,14 +102,10 @@ var expectedTLSConfigs = []struct {
 func TestValidTLSConfig(t *testing.T) {
 	for _, cfg := range expectedTLSConfigs {
 		got, err := LoadTLSConfig("testdata/" + cfg.filename)
-		if err != nil {
-			t.Fatalf("Error parsing %s: %s", cfg.filename, err)
-		}
+		require.NoErrorf(t, err, "Error parsing %s: %s", cfg.filename, err)
 		// non-nil functions are never equal.
 		got.GetClientCertificate = nil
-		if !reflect.DeepEqual(got, cfg.config) {
-			t.Fatalf("%v: unexpected config result: \n\n%v\n expected\n\n%v", cfg.filename, got, cfg.config)
-		}
+		require.Truef(t, reflect.DeepEqual(got, cfg.config), "%v: unexpected config result: \n\n%v\n expected\n\n%v", cfg.filename, got, cfg.config)
 	}
 }
 
@@ -127,8 +132,83 @@ func TestInvalidTLSConfig(t *testing.T) {
 	}
 }
 
-func TestStringer(t *testing.T) {
-	if s := (TLSVersion)(tls.VersionTLS13); s.String() != "TLS13" {
-		t.Fatalf("tls.VersionTLS13 string should be TLS13, got %s", s.String())
+func TestTLSVersionStringer(t *testing.T) {
+	s := (TLSVersion)(tls.VersionTLS13)
+	require.Equalf(t, "TLS13", s.String(), "tls.VersionTLS13 string should be TLS13, got %s", s.String())
+}
+
+func TestTLSVersionMarshalYAML(t *testing.T) {
+	tests := []struct {
+		input    TLSVersion
+		expected string
+		err      error
+	}{
+		{
+			input:    TLSVersions["TLS13"],
+			expected: "TLS13\n",
+			err:      nil,
+		},
+		{
+			input:    TLSVersions["TLS10"],
+			expected: "TLS10\n",
+			err:      nil,
+		},
+		{
+			input:    TLSVersion(999),
+			expected: "",
+			err:      errors.New("unknown TLS version: 999"),
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(fmt.Sprintf("MarshalYAML(%d)", test.input), func(t *testing.T) {
+			actualBytes, err := yaml.Marshal(&test.input)
+			if err != nil {
+				if test.err == nil || err.Error() != test.err.Error() {
+					t.Fatalf("error %v, expected %v", err, test.err)
+				}
+				return
+			}
+			actual := string(actualBytes)
+			require.Equalf(t, test.expected, actual, "returned %s, expected %s", actual, test.expected)
+		})
+	}
+}
+
+func TestTLSVersionMarshalJSON(t *testing.T) {
+	tests := []struct {
+		input    TLSVersion
+		expected string
+		err      error
+	}{
+		{
+			input:    TLSVersions["TLS13"],
+			expected: `"TLS13"`,
+			err:      nil,
+		},
+		{
+			input:    TLSVersions["TLS10"],
+			expected: `"TLS10"`,
+			err:      nil,
+		},
+		{
+			input:    TLSVersion(999),
+			expected: "",
+			err:      errors.New("unknown TLS version: 999"),
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(fmt.Sprintf("MarshalJSON(%d)", test.input), func(t *testing.T) {
+			actualBytes, err := json.Marshal(&test.input)
+			if err != nil {
+				if test.err == nil || !strings.HasSuffix(err.Error(), test.err.Error()) {
+					t.Fatalf("error %v, expected %v", err, test.err)
+				}
+				return
+			}
+			actual := string(actualBytes)
+			require.Equalf(t, test.expected, actual, "returned %s, expected %s", actual, test.expected)
+		})
 	}
 }
