@@ -24,7 +24,7 @@ import (
 func TestRedirect(t *testing.T) {
 	router := New().WithPrefix("/test/prefix")
 	w := httptest.NewRecorder()
-	r, err := http.NewRequest("GET", "http://localhost:9090/foo", nil)
+	r, err := http.NewRequest(http.MethodGet, "http://localhost:9090/foo", nil)
 	require.NoErrorf(t, err, "Error building test request: %s", err)
 
 	router.Redirect(w, r, "/some/endpoint", http.StatusFound)
@@ -37,20 +37,20 @@ func TestRedirect(t *testing.T) {
 
 func TestContext(t *testing.T) {
 	router := New()
-	router.Get("/test/:foo/", func(w http.ResponseWriter, r *http.Request) {
+	router.Get("/test/:foo/", func(_ http.ResponseWriter, r *http.Request) {
 		want := "bar"
 		got := Param(r.Context(), "foo")
 		require.Equalf(t, want, got, "Unexpected context value: want %q, got %q", want, got)
 	})
 
-	r, err := http.NewRequest("GET", "http://localhost:9090/test/bar/", nil)
+	r, err := http.NewRequest(http.MethodGet, "http://localhost:9090/test/bar/", nil)
 	require.NoErrorf(t, err, "Error building test request: %s", err)
 	router.ServeHTTP(nil, r)
 }
 
 func TestContextWithValue(t *testing.T) {
 	router := New()
-	router.Get("/test/:foo/", func(w http.ResponseWriter, r *http.Request) {
+	router.Get("/test/:foo/", func(_ http.ResponseWriter, r *http.Request) {
 		want := "bar"
 		got := Param(r.Context(), "foo")
 		require.Equalf(t, want, got, "Unexpected context value: want %q, got %q", want, got)
@@ -62,7 +62,7 @@ func TestContextWithValue(t *testing.T) {
 		require.Equalf(t, want, got, "Unexpected context value: want %q, got %q", want, got)
 	})
 
-	r, err := http.NewRequest("GET", "http://localhost:9090/test/bar/", nil)
+	r, err := http.NewRequest(http.MethodGet, "http://localhost:9090/test/bar/", nil)
 	require.NoErrorf(t, err, "Error building test request: %s", err)
 	params := map[string]string{
 		"lorem": "ipsum",
@@ -71,7 +71,7 @@ func TestContextWithValue(t *testing.T) {
 
 	ctx := r.Context()
 	for p, v := range params {
-		ctx = WithParam(ctx, p, v)
+		ctx = WithParam(ctx, p, v) //nolint:fatcontext
 	}
 	r = r.WithContext(ctx)
 	router.ServeHTTP(nil, r)
@@ -79,13 +79,13 @@ func TestContextWithValue(t *testing.T) {
 
 func TestContextWithoutValue(t *testing.T) {
 	router := New()
-	router.Get("/test", func(w http.ResponseWriter, r *http.Request) {
+	router.Get("/test", func(_ http.ResponseWriter, r *http.Request) {
 		want := ""
 		got := Param(r.Context(), "foo")
 		require.Equalf(t, want, got, "Unexpected context value: want %q, got %q", want, got)
 	})
 
-	r, err := http.NewRequest("GET", "http://localhost:9090/test", nil)
+	r, err := http.NewRequest(http.MethodGet, "http://localhost:9090/test", nil)
 	require.NoErrorf(t, err, "Error building test request: %s", err)
 	router.ServeHTTP(nil, r)
 }
@@ -109,9 +109,9 @@ func TestInstrumentation(t *testing.T) {
 	}
 
 	for _, c := range cases {
-		c.router.Get("/foo", func(w http.ResponseWriter, r *http.Request) {})
+		c.router.Get("/foo", func(_ http.ResponseWriter, _ *http.Request) {})
 
-		r, err := http.NewRequest("GET", "http://localhost:9090/foo", nil)
+		r, err := http.NewRequest(http.MethodGet, "http://localhost:9090/foo", nil)
 		require.NoErrorf(t, err, "Error building test request: %s", err)
 		c.router.ServeHTTP(nil, r)
 		require.Equalf(t, c.want, got, "Unexpected value: want %q, got %q", c.want, got)
@@ -149,9 +149,9 @@ func TestInstrumentations(t *testing.T) {
 	}
 
 	for _, c := range cases {
-		c.router.Get("/foo", func(w http.ResponseWriter, r *http.Request) {})
+		c.router.Get("/foo", func(_ http.ResponseWriter, _ *http.Request) {})
 
-		r, err := http.NewRequest("GET", "http://localhost:9090/foo", nil)
+		r, err := http.NewRequest(http.MethodGet, "http://localhost:9090/foo", nil)
 		require.NoErrorf(t, err, "Error building test request: %s", err)
 		c.router.ServeHTTP(nil, r)
 		require.Lenf(t, got, len(c.want), "Unexpected value: want %q, got %q", c.want, got)
